import { useQuery } from 'urql';
import gql from 'graphql-tag';
import ErrorMessage from './ErrorMessage';
import PostUpvoter from './PostUpvoter';

const allPostsQuery = gql`
  query allPosts($first: Int!, $skip: Int!) {
    allPosts(orderBy: createdAt_DESC, first: $first, skip: $skip) {
      id
      title
      votes
      url
      createdAt
    }
    _allPostsMeta {
      count
    }
  }
`;

const allPostsQueryVars = {
  skip: 0,
  first: 10,
};

export default function PostList() {
  const [allPostsResult] = useQuery({
    query: allPostsQuery,
    variables: allPostsQueryVars,
  });

  if (allPostsResult.error) {
    return <ErrorMessage message="Error loading posts." />;
  } else if (allPostsResult.loading || !allPostsResult.data) {
    return <div>Loading</div>;
  }

  const { allPosts, _allPostsMeta } = allPostsResult.data;

  return (
    <section>
      <ul>
        {allPosts.map((post, index) => (
          <li key={post.id}>
            <div>
              <span>{index + 1}. </span>
              <a href={post.url}>{post.title}</a>
              <PostUpvoter id={post.id} votes={post.votes} />
            </div>
          </li>
        ))}
      </ul>

      <style jsx>{`
        section {
          padding-bottom: 20px;
        }
        li {
          display: block;
          margin-bottom: 10px;
        }
        div {
          align-items: center;
          display: flex;
        }
        a {
          font-size: 14px;
          margin-right: 10px;
          text-decoration: none;
          padding-bottom: 0;
          border: 0;
        }
        span {
          font-size: 14px;
          margin-right: 5px;
        }
        ul {
          margin: 0;
          padding: 0;
        }
        button:before {
          align-self: center;
          border-style: solid;
          border-width: 6px 4px 0 4px;
          border-color: #ffffff transparent transparent transparent;
          content: '';
          height: 0;
          margin-right: 5px;
          width: 0;
        }
      `}</style>
    </section>
  );
}
